var gulp = require('gulp');
var concat = require('gulp-concat');
var babel = require("gulp-babel");
var uglify = require('gulp-uglify');
var rename = require('gulp-rename');
var bump = require('gulp-bump');
var git = require('gulp-git');
var runSequence = require('run-sequence');
var publishRelease = require('publish-release');
var header = require('gulp-header');
var clean = require('gulp-clean');
var gulpWebpack = require('webpack-stream');
var webpack = require('webpack');
var gzipSize = require('gzip-size');

var fs = require('fs');

gulp.task("clean", function(){
  return gulp.src("bin/", {read: false})
    .pipe(clean());
});

gulp.task("babel", function () {
  return babelFunc();
});

function babelFunc(){
  return gulp.src("src/**/*")
    .pipe(babel({
      presets: ['es2015'],
      plugins: [["transform-es2015-classes", {loose: true}]]
    }))
    .pipe(gulp.dest("bin/node/"));
}

gulp.task("webpack", ["babel"], function () {
  return webpackFunc();
});

function webpackFunc(){
  return gulp.src('bin/node/JsBarcode.js')
    .pipe(gulpWebpack(
      {
        output: {
          filename: 'JsBarcode.all.js',
        }
      }
    , webpack))
    .pipe(gulp.dest("bin/browser/"));
}

gulp.task("webpack-min", ["babel"], function () {
  return webpackMin('JsBarcode.all.min.js');
});

function webpackMin(filename){
  return gulp.src('bin/node/JsBarcode.js')
    .pipe(gulpWebpack(
      {
        output: {
          filename: filename,
        },
        plugins: [new webpack.optimize.UglifyJsPlugin()],
      }
    , webpack))
    .pipe(gulp.dest("bin/browser/"));
}

gulp.task("webpack-all", function (cb) {
  var barcodes = require('./barcode-building.json');

  fs.renameSync("src/barcodes/index.js", "src/barcodes/index.tmp.js");

  function createBarcodeInclude(barcode, callback){
    var toFile = "";
    toFile += "import {" + barcode.names + "} from '" + barcode.barcodeFile + "'";
    toFile += "\n";
    toFile += "export default {" + barcode.names + "}";

    fs.writeFile("src/barcodes/index.js", toFile, function(){
      if(fs.existsSync("bin/node/barcodes/index.js")){
        fs.unlinkSync("bin/node/barcodes/index.js");
      }
      babelFunc().on('end', function(){
        webpackMin("barcodes/" + barcode.filename).on('end', callback);
      });
    });
  }

  var i = 0;
  (function loopBarcodes(){
    createBarcodeInclude(barcodes[i], function(){
      i++;
      if(i < barcodes.length){
        loopBarcodes();
      }
      else{
        fs.renameSync("src/barcodes/index.tmp.js", "src/barcodes/index.js");
        cb(); // Done
      }
    });
  })();
});

gulp.task('watch', ['compile'], function() {
  gulp.watch("src/**/*", ['compile']);
});

gulp.task('watch-web', ['webpack'], function() {
  gulp.watch("src/**/*", ['webpack']);
});

gulp.task('compress', function(cb) {
  runSequence(
    "clean",
    "webpack-all",
    "webpack",
    "webpack-min",
    cb
  );
});


gulp.task('git-release', ['compress'], function(cb){
  var pkg = require('./package.json');
  var v = 'v' + pkg.version;
  var message = 'Release ' + v;

  updateReadmeFileSizes();

  gulp.src(['./package.json', './bower.json', './bin/'])
    .pipe(git.add({args: '--all'}))
    .pipe(git.commit(message));

  git.push('origin', 'master', function(){
    git.tag(v, message, function(){
      git.push('origin', 'master', {args: '--tags'}, cb);
    });
  });
});

// Bump (increase) the version number
gulp.task('bump-patch', function(){
  return gulp.src(['./package.json', './bower.json'])
    .pipe(bump({type:'patch'}))
    .pipe(gulp.dest('./'));
});

gulp.task('bump-minor', function(){
  return gulp.src(['./package.json', './bower.json'])
    .pipe(bump({type:'minor'}))
    .pipe(gulp.dest('./'));
});

gulp.task('bump-major', function(){
  return gulp.src(['./package.json', './bower.json'])
    .pipe(bump({type:'major'}))
    .pipe(gulp.dest('./'));
});

gulp.task('npm', function (done) {
  require('child_process').spawn('npm', ['publish'], { stdio: 'inherit' })
    .on('close', done);
});

gulp.task('github-release', function(done) {
  var pkg = require('./package.json');
  var v = 'v' + pkg.version;
  var name = "JsBarcode " + v;

  publishRelease({
    token: process.env.GITHUB_TOKEN,
    owner: "lindell",
    repo: "JsBarcode",
    tag: v,
    name: name,
    assets: [__dirname + "/bin/browser/JsBarcode.all.min.js", __dirname + "/bin/browser/JsBarcode.all.js"]
  }, done);
});

function updateReadmeFileSizes(){
  var files = require('./barcode-building.json');
  var readme = fs.readFileSync('README.md', "utf-8");

  // Update .all files
  var data = fs.readFileSync('bin/browser/JsBarcode.all.min.js');
  var filesize = gzipSize.sync(data);

  var regexp = new RegExp('\\|[^\\|]*\\|([ \\t\\*]*\\[JsBarcode\\.all\\.min\\.js\\])');
  readme = readme.replace(regexp, "|  *" + formatSize(filesize) + "*  |$1");

  // Update all barcodes files
  for(var i in files){
    var filename = files[i].filename;

    var data = fs.readFileSync('bin/browser/barcodes/' + filename);
    var filesize = gzipSize.sync(data);

    var regexp = new RegExp('\\|[^\\|]*\\|([ \\t]*\\[' + RegExp.escape(filename) + '\\])');

    readme = readme.replace(regexp, "|  " + formatSize(filesize) + "  |$1");
  }

  fs.writeFileSync('README.md', readme, 'utf8');
}

function formatSize(bytes){
  var kilobytes = Math.round(bytes/1024*10)/10;

  return kilobytes + " kB";
}

// Needed so that github can register the push before new release
gulp.task('wait', function(done) {
  setTimeout(done, 5000);
});

var done = function (error) {
  if (error) {
    console.log(error.message);
  }
  else {
    console.log('Successful!');
  }
};


gulp.task('compile', ['babel'], function(done){

});

gulp.task('compile-web', ['webpack'], function(done){

});

gulp.task('release', function(callback){
  runSequence(
    'git-release',
    'wait',
    'github-release',
    'npm',
    callback
  );
});

gulp.task('patch', function(){
  runSequence(
    'bump-patch',
    'release',
    done
  );
});

gulp.task('minor', function(){
  runSequence(
    'bump-minor',
    'release',
    done
  );
});

gulp.task('major', function(){
  runSequence(
    'bump-major',
    'release',
    done
  );
});

// Util functions

RegExp.escape= function(s) {
    return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
};
